#ifndef FUNCTOR_MD
#define FUNCTOR_MD

/**
 * @file FunctorMD.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @date October 07, 2016
 * @version 1.0
 */

#include <vector>

#include "../../linear_algebra/vector/VectorD.h"
#include "FunctionTypeMD.h"

namespace NumA {

/**
 * @class FunctorMD
 *
 * @brief Template class for defining multi-dimensional functions that can be used as arguments in virtual methods.
 * This is needed to circumvent the issue that templates and virtual methods are incompatible.
 *
 * See FunctionTypeMD documentation for usage.
 */

template<typename PointerToObj, typename PointerToFunc>
class FunctorMD: public FunctionTypeMD {
public:
    /**
     * Constructor.
     * @param object Pointer to an object instantiating the class where the function is defined.
     * @param function Reference to the function.
     */
    FunctorMD(PointerToObj* object, PointerToFunc function) :
            m_pObject(object), m_pFunction(function) {
    }

    /**
     * Default destructor.
     */
    virtual ~FunctorMD() {
    }

    inline double operator()(VectorD &variables,
            std::vector<double> &parameters) {
        return ((*m_pObject).*m_pFunction)(variables, parameters);
    }
    inline double operator()(VectorD &variables) {
        std::vector<double> parameters;
        return ((*m_pObject).*m_pFunction)(variables, parameters);
    }

private:
    PointerToObj* m_pObject; ///< Pointer to an object instantiating the class where the function is defined.
    PointerToFunc m_pFunction; ///< Reference to the function.

    FunctorMD(const FunctorMD&); ///< Not implemented. Disallows copy.
    FunctorMD& operator=(const FunctorMD&); ///< Not implemented. Disallows comparisons.
};

} // namespace NumA

#endif /* FUNCTOR_MD */
