#ifndef FUNCTOR_1D
#define FUNCTOR_1D

/**
 * @file Functor1D.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @date 15 February 2016
 * @version 1.0
 */

#include <vector>

#include "FunctionType1D.h"

namespace NumA {

/**
 * @class Functor1D
 *
 * @brief Template class for defining one-dimensional functions that can be used as arguments in virtual methods.
 * This is needed to circumvent the issue that templates and virtual methods are incompatible.
 *
 * See FunctionType1D documentation for usage.
 */

template<typename PointerToObj, typename PointerToFunc>
class Functor1D: public FunctionType1D {
public:
    /**
     * Constructor.
     * @param object Pointer to an object instantiating the class where the function is defined.
     * @param function Reference to the function.
     */
    Functor1D(PointerToObj* object, PointerToFunc function) :
            m_pObject(object), m_pFunction(function) {
    }

    /**
     * Default destructor.
     */
    virtual ~Functor1D() {
    }

    inline double operator()(double x, std::vector<double> &parameters) {
        return ((*m_pObject).*m_pFunction)(x, parameters);
    }
    inline double operator()(double x) {
        std::vector<double> parameters;
        return ((*m_pObject).*m_pFunction)(x, parameters);
    }

private:
    PointerToObj* m_pObject; ///< Pointer to an object instantiating the class where the function is defined.
    PointerToFunc m_pFunction; ///< Reference to the function.

    Functor1D(const Functor1D&); ///< Not implemented. Disallows copy.
    Functor1D& operator=(const Functor1D&); ///< Not implemented. Disallows comparisons.
};

} // namespace NumA

#endif /* FUNCTOR_1D */
