#ifndef CHEBYSHEVA_INTEGRATOR_1D_H
#define CHEBYSHEVA_INTEGRATOR_1D_H

/**
 * @file ChebyshevAIntegrator1D.h
 * @author Daniele BINOSI (ECT*)
 * @date 27 April 2016
 * @version 1.0
 */

//#include <string>
#include "QuadratureIntegrator1D.h"

class FunctionType1D;

namespace NumA {

/**
 * @class ChebyshevAIntegrator1D
 *
 * @brief Chebyshev quadrature for computing the integral \f$ \int_{-1}^1 \mathrm{dx} f(x) \sqrt{1-x^2} \f$.
 *
 * See Integrator1D documentation for an example.
 */

class ChebyshevAIntegrator1D: public QuadratureIntegrator1D {
public:
    ChebyshevAIntegrator1D(unsigned int N = 48);
    virtual ~ChebyshevAIntegrator1D();

    virtual ChebyshevAIntegrator1D* clone() const;

    virtual void setN(unsigned int n);

    // Not needed anymore, done in parent class.
//    virtual double integrate(FunctionType1D* pFunction, double a, double b,
//            std::vector<double>& parameters);

protected:
    ChebyshevAIntegrator1D(const ChebyshevAIntegrator1D &other);
    void setNodesAndWeights();
};

} /* namespace NumA */

#endif /* CHEBYSHEVA_INTEGRATOR_1D_H */
