#ifndef CHEBYSHEVB_INTEGRATOR_1D_H
#define CHEBYSHEVB_INTEGRATOR_1D_H

/**
 * @file ChebyshevBIntegrator1D.h
 * @author Daniele BINOSI (ECT*)
 * @date 27 April 2016
 * @version 1.0
 */

//#include <string>
#include "QuadratureIntegrator1D.h"

class FunctionType1D;

namespace NumA {

/**
 * @class ChebyshevBIntegrator1D
 *
 * @brief Chebyshev quadrature for computing the integral \f$ \int_{-1}^1 \mathrm{dx} \frac{f(x)}{\sqrt{1-x^2}} \f$.
 *
 * See Integrator1D documentation for an example.
 */

class ChebyshevBIntegrator1D: public QuadratureIntegrator1D {
public:
    ChebyshevBIntegrator1D(unsigned int N = 48);
    virtual ~ChebyshevBIntegrator1D();

    virtual ChebyshevBIntegrator1D* clone() const;

    virtual void setN(unsigned int n);

    // Not needed anymore, done in parent class.
//    virtual double integrate(FunctionType1D* pFunction, double a, double b,
//            std::vector<double>& parameters);

protected:
    ChebyshevBIntegrator1D(const ChebyshevBIntegrator1D &other);
    void setNodesAndWeights();
};

} /* namespace NumA */

#endif /* CHEBYSHEVB_INTEGRATOR_1D_H */
