#ifndef GAUSSLEGENDRE_INTEGRATOR_1D_H_
#define GAUSSLEGENDRE_INTEGRATOR_1D_H_

/**
 * @file GaussLegendreIntegrator1D.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @author Michel Guidal and Pawel Sznajder (IPNO)
 * @author Cédric Mezrag (ANL / PHY)
 * @author Nabil Chouika (SPhN / CEA Saclay)
 * @date 21 December 2015
 * @version 1.0
 */

//#include <string>
#include "QuadratureIntegrator1D.h"

namespace NumA {

/**
 * @class GaussLegendreIntegrator1D
 *
 * @brief Gauss-Legendre quadrature.
 *
 * See Integrator1D documentation for an example.
 */

class GaussLegendreIntegrator1D: public QuadratureIntegrator1D {
public:
    /**
     * Constructor.
     * @param N Order of the quadrature (number of nodes).
     */
    GaussLegendreIntegrator1D(unsigned int N = 50); // TODO how many steps by default ?
    /**
     * Default destructor.
     */
    virtual ~GaussLegendreIntegrator1D();

    virtual void setN(unsigned int n);

    virtual GaussLegendreIntegrator1D* clone() const;

protected:
    /**
     * Copy constructor.
     * Called by clone().
     * @param other GaussLegendreIntegrator1D object to copy.
     */
    GaussLegendreIntegrator1D(const GaussLegendreIntegrator1D &other);
    void setNodesAndWeights(); ///< Computes the nodes and weights of the Gauss-Legendre quadrature. Called when N is set.
};

} /* namespace NumA */

#endif /* GAUSSLEGENDRE_INTEGRATOR_1D_H_ */
