#ifndef GAUSSLEGENDRESEST_INTEGRATOR_1D_H_
#define GAUSSLEGENDRESEST_INTEGRATOR_1D_H_

/**
 * @file GaussLegendreSeStIntegrator1D.h
 * @author Daniele BINOSI (ECT*)
 * @date 6 May 2016
 * @version 1.0
 */

//#include <string>
#include "QuadratureIntegrator1D.h"

namespace NumA {

/**
 * @class GaussLegendreSeStIntegrator1D
 *
 * @brief Gauss-Legendre quadrature.
 *
 * See Integrator1D documentation for an example.
 */

class GaussLegendreSeStIntegrator1D: public QuadratureIntegrator1D {
public:
    /**
     * Constructor.
     * @param N Order of the quadrature (number of nodes).
     */
    GaussLegendreSeStIntegrator1D(unsigned int N = 50); // TODO how many steps by default ?
    /**
     * Default destructor.
     */
    virtual ~GaussLegendreSeStIntegrator1D();

    virtual void setN(unsigned int n);

    virtual GaussLegendreSeStIntegrator1D* clone() const;

protected:
    /**
     * Copy constructor.
     * Called by clone().
     * @param other GaussLegendreSeStIntegrator1D object to copy.
     */
    GaussLegendreSeStIntegrator1D(const GaussLegendreSeStIntegrator1D &other);
    void setNodesAndWeights(); ///< Computes the nodes and weights of the Gauss-Legendre quadrature. Called when N is set.
};

} /* namespace NumA */

#endif /* GAUSSLEGENDRE_INTEGRATOR_1D_H_ */
