#ifndef TRAPEZOIDAL_INTEGRATOR_1D_H
#define TRAPEZOIDAL_INTEGRATOR_1D_H

/**
 * @file TrapezoidalIntegrator1D.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @author Cédric Mezrag (ANL / PHY)
 * @date 23 October 2015
 * @version 1.0
 */

#include "QuadratureIntegrator1D.h"

class FunctionType1D;

namespace NumA {

/**
 * @class TrapezoidalIntegrator1D
 *
 * @brief Trapezoidal integration.
 *
 * See Integrator1D documentation for an example.
 */

class TrapezoidalIntegrator1D: public QuadratureIntegrator1D {

public:

    /**
     * Constructor.
     * @param N Order of the quadrature (number of nodes).
     */
    TrapezoidalIntegrator1D(unsigned int N = 40);

    /**
     * Default destructor.
     */
    virtual ~TrapezoidalIntegrator1D();

    virtual TrapezoidalIntegrator1D* clone() const;

    virtual void setN(unsigned int n);

protected:

    /**
     * Copy constructor.
     * Called by clone().
     * @param other TrapezoidalIntegrator1D object to copy.
     */
    TrapezoidalIntegrator1D(const TrapezoidalIntegrator1D &other);

    /**
     * Computes the nodes and weights of the Gauss-Legendre quadrature. Called when N is set.
     */
    void setNodesAndWeights();
};

} /* namespace NumA */

#endif /* TRAPEZOIDAL_INTEGRATOR_1D_H */
