#ifndef TRAPEZOIDAL_LOG_INTEGRATOR_1D_H
#define TRAPEZOIDAL_LOG_INTEGRATOR_1D_H

/**
 * @file TrapezoidalLogIntegrator1D.h
 * @author Pawel Sznajder (NCBJ, Warsaw)
 * @date 12 July 2017
 * @version 1.0
 */

#include <ElementaryUtils/parameters/Parameters.h>
#include <include/NumA/integration/one_dimension/QuadratureIntegrator1D.h>
#include <string>
#include <vector>

class FunctionType1D;

namespace NumA {

/**
 * @class TrapezoidalLogIntegrator1D
 *
 * @brief Trapezoidal integration with logarithmic step.
 *
 * See Integrator1D documentation for an example.
 */

class TrapezoidalLogIntegrator1D: public QuadratureIntegrator1D {

public:

	/**
	 * Constructor.
	 * @param N Order of the quadrature (number of nodes).
	 */
	TrapezoidalLogIntegrator1D(unsigned int N = 40);

	/**
	 * Default destructor.
	 */
	virtual ~TrapezoidalLogIntegrator1D();

	virtual TrapezoidalLogIntegrator1D* clone() const;
	virtual void configure(const ElemUtils::Parameters &parameters);
	virtual void setN(unsigned int n);
	virtual double integrate(FunctionType1D* pFunction, double a, double b,
			std::vector<double> &parameters);

protected:

	/**
	 * Copy constructor.
	 * Called by clone().
	 * @param other TrapezoidalLogIntegrator1D object to copy.
	 */
	TrapezoidalLogIntegrator1D(const TrapezoidalLogIntegrator1D &other);
};

} /* namespace NumA */

#endif /* TRAPEZOIDAL_LOG_INTEGRATOR_1D_H */
