/**
 * @file CubicSpline.h
 * @author Cédric Mezrag (ANL)
 * @date 14 April 2017
 * @version 0.1
 */

#ifndef CUBICSPLINE_H_
#define CUBICSPLINE_H_

#include <iostream>
#include <sstream>
#include <vector>
#include <cmath>



/*
 * Cubic spline algorithm based on wikipedia :
 * https://en.wikipedia.org/w/index.php?title=Spline_%28mathematics%29&oldid=288288033#Algorithm_for_computing_natural_cubic_splines
 *
 *
 */

namespace NumA{

class CubicSpline{

public:

	CubicSpline(std::vector<double> x, std::vector<double> y);
	virtual ~CubicSpline();

	void ConstructSpline(); // construct natural cubic splines

	double getSplineInsideValue(double z); //Return natural cubic splines for xmin < z < xmax and 0 if z is outside the set of point.

private:

	const unsigned int m_N ; // size of the point sample

	std::vector<double> m_X ;  // x coordinate of the sample
	std::vector<double> m_Y ;  // y coordinate of the sample

	std::vector<double> m_aCoeff ; // coefficient a as defined on wikipedia
	std::vector<double> m_bCoeff ;	// coefficient b as defined on wikipedia
	std::vector<double> m_cCoeff ; // coefficient c as defined on wikipedia
	std::vector<double> m_dCoeff ; // coefficient d as defined on wikipedia

	bool m_SplineDefined ; // Turn to true if ConstructSpline is run at least once.

};


}


#endif
