/**
 * @file LinAlgUtils.h
 * @author Nabil CHOUIKA (SPhN / CEA Saclay)
 * @date Feb 4 2016
 * @version 1.0
 */

#ifndef LINALGUTILS_H_
#define LINALGUTILS_H_

#include "matrix/MatrixD.h"
#include "vector/VectorD.h"

namespace ElemUtils {
class Formatter;
} /* namespace ElemUtils */

namespace NumA {
class MatrixD;
} /* namespace NumA */

namespace NumA {

/**
 * @class LinAlgUtils
 * @brief Linear algebra routines such as linear solvers. Serves as wrapper for Eigen.
 */

class LinAlgUtils {
public:
    enum Method {
        ColQR = 0, FullQR, QR, PartialLU, FullLU
    };

    /** @brief Solves the linear system \f$ A X = Y \f$ and returns \f$ X \f$ .
     *
     * @param A Matrix to invert.
     * @param Y Right hand side.
     * @param method Solver method.
     * @return X Solution: VectorD
     */
    static VectorD solve(const MatrixD & A, const VectorD & Y, Method method =
            ColQR);

    /**
     * @brief Computes the rank of the matrix with the given method.
     * @param A MatrixD
     * @param method
     * @return rank unsigned int
     */
    static unsigned int rank(const MatrixD & A, Method method = ColQR);
};

} // namespace NumA

#endif /* LINALGUTILS_H_ */
