/**
 * @file EigenUtils.h
 * @author Nabil Chouika (Irfu/SPhN, CEA Saclay)
 * @date 11 oct. 2016
 * @version 1.0
 */

#ifndef EIGENUTILS_H_
#define EIGENUTILS_H_

#include <Eigen/Core>

#include "../matrix/MatrixD.h"
#include "../vector/VectorD.h"

namespace NumA {

/**
 * @class EigenUtils
 * @brief Tools for the Eigen wrapper.
 *
 * Workaround to not have PARTONS depend on Eigen.
 * The methods defined here are only used in the cpp files of NumA++. Do not include this file in PARTONS.
 */

class EigenUtils {
public:
    /**
     * Convert a matrix from NumA type to Eigen type.
     * @param A NumA matrix.
     * @return Eigen matrix.
     */
    static Eigen::MatrixXd convertToEigen(const MatrixD & A);
    /**
     * Convert a vector from NumA type to Eigen type.
     * @param V NumA vector.
     * @return Eigen vector.
     */
    static Eigen::VectorXd convertToEigen(const VectorD & V);
    /**
     * Convert a vector from Eigen type to NumA type.
     * @param V Eigen vector.
     * @return NumA vector.
     */
    static VectorD convertToVectorD(const Eigen::VectorXd & V);
};

} /* namespace NumA */

#endif /* EIGENUTILS_H_ */
