/*
 * MatrixComplex2D.h
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#ifndef MATRIXCOMPLEX2D_H_
#define MATRIXCOMPLEX2D_H_

#include <complex>

#include "MatrixComplexD.h"

namespace NumA {

/**
 * @class MatrixComplex2D
 *
 * @brief Matrix of complex numbers of size 2x2.
 *
 * This class represents a matrix of complex double precision numbers. The size of this matrix is 2x2.
 */
class MatrixComplex2D: public MatrixComplexD {

public:

    /**
     * Default constructor.
     */
    MatrixComplex2D();

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    MatrixComplex2D(const MatrixComplex2D& v);

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    MatrixComplex2D(const MatrixComplexD& v);

    /**
     * Destructor.
     */
    ~MatrixComplex2D();

    /**
     * Operator =.
     */
    void operator=(const MatrixComplex2D& rhs);

    /**
     * Operator +.
     */
    MatrixComplex2D operator+(const MatrixComplex2D& rhs);

    /**
     * Operator -.
     */
    MatrixComplex2D operator-(const MatrixComplex2D& rhs);

    /**
     * Operator * by real number (rhs).
     */
    MatrixComplex2D operator*(const double& rhs);

    /**
     * Operator * by real number (lhs).
     */
    friend MatrixComplex2D operator*(const double& lhs, MatrixComplex2D& rhs);

    /**
     * Operator * by complex number (rhs).
     */
    MatrixComplex2D operator*(const std::complex<double>& rhs);

    /**
     * Operator * by complex number (lhs).
     */
    friend MatrixComplex2D operator*(const std::complex<double>& lhs,
            MatrixComplex2D& rhs);
};

} /* namespace NumA */

#endif /* MATRIXCOMPLEXD_H_ */
