/*
 * MatrixComplex3D.h
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#ifndef MATRIXCOMPLEX3D_H_
#define MATRIXCOMPLEX3D_H_

#include <complex>

#include "MatrixComplexD.h"

namespace NumA {

/**
 * @class MatrixComplex3D
 *
 * @brief Matrix of complex numbers of size 3x3.
 *
 * This class represents a matrix of complex double precision numbers. The size of this matrix is 3x3.
 */
class MatrixComplex3D: public MatrixComplexD {

public:

    /**
     * Default constructor.
     */
    MatrixComplex3D();

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    MatrixComplex3D(const MatrixComplex3D& v);

    /** Copy constructor.
     * @param v Object to be copied.
     */
    MatrixComplex3D(const MatrixComplexD& v);

    /**
     * Destructor.
     */
    ~MatrixComplex3D();

    /**
     * Operator =.
     */
    void operator=(const MatrixComplex3D& rhs);

    /**
     * Operator +.
     */
    MatrixComplex3D operator+(const MatrixComplex3D& rhs);

    /**
     * Operator -.
     */
    MatrixComplex3D operator-(const MatrixComplex3D& rhs);

    /**
     * Operator * by real number (rhs).
     */
    MatrixComplex3D operator*(const double& rhs);

    /**
     * Operator * by real number (lhs).
     */
    friend MatrixComplex3D operator*(const double& lhs, MatrixComplex3D& rhs);

    /**
     * Operator * by complex number (rhs).
     */
    MatrixComplex3D operator*(const std::complex<double>& rhs);

    /**
     * Operator * by complex number (lhs).
     */
    friend MatrixComplex3D operator*(const std::complex<double>& lhs,
            MatrixComplex3D& rhs);
};

} /* namespace NumA */

#endif /* MATRIXCOMPLEXD_H_ */
