/*
 * MatrixComplex4D.h
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#ifndef MATRIXCOMPLEX4D_H_
#define MATRIXCOMPLEX4D_H_

#include <complex>

#include "MatrixComplexD.h"

namespace NumA {

/**
 * @class MatrixComplex4D
 *
 * @brief Matrix of complex numbers of size 4x4.
 *
 * This class represents a matrix of complex double precision numbers. The size of this matrix is 4x4.
 */
class MatrixComplex4D: public MatrixComplexD {

public:

    /**
     * Default constructor.
     */
    MatrixComplex4D();

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    MatrixComplex4D(const MatrixComplex4D& v);

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    MatrixComplex4D(const MatrixComplexD& v);

    /**
     * Destructor.
     */
    ~MatrixComplex4D();

    /**
     * Operator =.
     */
    void operator=(const MatrixComplex4D& rhs);

    /**
     * Operator +.
     */
    MatrixComplex4D operator+(const MatrixComplex4D& rhs);

    /**
     * Operator -.
     */
    MatrixComplex4D operator-(const MatrixComplex4D& rhs);

    /**
     * Operator * by real number (rhs).
     */
    MatrixComplex4D operator*(const double& rhs);

    /**
     * Operator * by real number (lhs).
     */
    friend MatrixComplex4D operator*(const double& lhs, MatrixComplex4D& rhs);

    /**
     * Operator * by complex number (rhs).
     */
    MatrixComplex4D operator*(const std::complex<double>& rhs);

    /**
     * Operator * by complex number (lhs).
     */
    friend MatrixComplex4D operator*(const std::complex<double>& lhs,
            MatrixComplex4D& rhs);
};

} /* namespace NumA */

#endif /* MATRIXCOMPLEXD_H_ */
