#ifndef VECTOR_2D_H
#define VECTOR_2D_H

/**
 * @file Vector2D.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @date 26 November 2014
 * @version 1.0
 */

//TODO mathematique function add, sub, ...
//TODO verifier les operations arithmetic
namespace NumA {

/**
 * @class Vector2D
 *
 * Object representing a two-dimensional vector.
 */
class Vector2D {
public:
    /**
     * Default constructor.
     */
    Vector2D();
    /**
     * Constructor.
     * @param x x-coordinate.
     * @param y y-coordinate.
     */
    Vector2D(double x, double y);
    /**
     * Default destructor.
     */
    ~Vector2D();

    // ##### GETTERS & SETTERS #####

    /**
     *
     * @return x-coordinate.
     */
    double getX() const;
    /**
     *
     * @param x x-coordinate.
     */
    void setX(double x);
    /**
     *
     * @return y-coordinate.
     */
    double getY() const;
    /**
     *
     * @param y y-coordinate.
     */
    void setY(double y);

    /**
     * Addition of two vectors.
     * @param rhs Vector2D.
     * @return Sum of two Vector2D.
     */
    Vector2D operator+(Vector2D const &rhs);
    /**
     * Addition with a scalar.
     * @param rhs Scalar.
     */
    void operator+=(Vector2D const &rhs);

    /**
     * Subtraction of two vectors
     * @param rhs Vector2D.
     * @return Subtraction of two Vector2D.
     */
    Vector2D operator-(Vector2D const &rhs);
    /**
     * Subtraction with a scalar.
     * @param rhs Scalar.
     */
    void operator-=(Vector2D const &rhs);

private:
    double m_x; ///< x-coordinate.
    double m_y; ///< y-coordinate.
};

} /* namespace NumA */

#endif /* VECTOR_2D_H */
