#ifndef VECTOR_3D_H
#define VECTOR_3D_H

/**
 * @file Vector3D.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @date 26 November 2014
 * @version 1.0
 */

#include <string>

#include "Vector2D.h"

namespace NumA {

//TODO mathematique function add, sub, ...

/**
 * @class Vector3D
 *
 * Object representing a three-dimensional vector.
 */
class Vector3D: public Vector2D {
public:
    /**
     * Default constructor.
     */
    Vector3D();
    /**
     * Constructor.
     * @param x x-coordinate.
     * @param y y-coordinate.
     * @param z z-coordinate.
     */
    Vector3D(double x, double y, double z);
    /**
     * Default destructor.
     */
    ~Vector3D();

    // Scalar operation
    /**
     * Scalar product.
     * @param rhs Vector3D.
     * @return double
     */
    double operator*(const Vector3D &rhs) const;

    // ##### GETTERS & SETTERS #####

    /**
     *
     * @return z-coordinate.
     */
    double getZ() const;
    /**
     *
     * @param z z-coordinate.
     */
    void setZ(double z);

    /**
     * Return a formatted characters string to display vector's values.
     *
     * @return std::string
     */
    std::string toString() const;

private:
    double m_z; ///< z-coordinate.
};

} /* namespace NumA */

#endif /* VECTOR_3D_H */
