/*
 * VectorComplex2D.h
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#ifndef VECTORCOMPLEX2D_H_
#define VECTORCOMPLEX2D_H_

#include <complex>

#include "VectorComplexD.h"

namespace NumA {

/**
 * @class VectorComplex2D
 *
 * @brief Vector of complex numbers of size 2.
 *
 * This class represents a vector of complex double precision numbers. The size of this vector is 2.
 */
class VectorComplex2D: public VectorComplexD {

public:

    /**
     * Default constructor.
     */
    VectorComplex2D();

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    VectorComplex2D(const VectorComplex2D& v);

    /** Copy constructor.
     * @param v Object to be copied.
     */
    VectorComplex2D(const VectorComplexD& v);

    /**
     * Destructor.
     */
    ~VectorComplex2D();

    /**
     * Operator =.
     */
    void operator=(const VectorComplex2D& v);

    /**
     * Operator +.
     */
    VectorComplex2D operator+(const VectorComplex2D& rhs);

    /**
     * Operator -.
     */
    VectorComplex2D operator-(const VectorComplex2D& rhs);

    /**
     * Operator * by real number (rhs).
     */
    VectorComplex2D operator*(const double& rhs);

    /**
     * Operator * by real number (lhs).
     */
    friend VectorComplex2D operator*(const double& lhs, VectorComplex2D& rhs);

    /**
     * Operator * by complex number (rhs).
     */
    VectorComplex2D operator*(const std::complex<double>& rhs);

    /**
     * Operator * by complex number (lhs).
     */
    friend VectorComplex2D operator*(const std::complex<double>& lhs,
            VectorComplex2D& rhs);
};

} /* namespace NumA */

#endif /* VECTORCOMPLEX2D_H_ */
