/*
 * VectorComplex3D.h
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#ifndef VECTORCOMPLEX3D_H_
#define VECTORCOMPLEX3D_H_

#include <complex>

#include "VectorComplexD.h"

namespace NumA {

/**
 * @class VectorComplex3D
 *
 * @brief Vector of complex numbers of size 3.
 *
 * This class represents a vector of complex double precision numbers. The size of this vector is 3.
 */
class VectorComplex3D: public VectorComplexD {

public:

    /**
     * Default constructor.
     */
    VectorComplex3D();

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    VectorComplex3D(const VectorComplex3D& v);

    /** Copy constructor.
     * @param v Object to be copied.
     */
    VectorComplex3D(const VectorComplexD& v);

    /**
     * Destructor.
     */
    ~VectorComplex3D();

    /**
     * Operator =.
     */
    virtual void operator=(const VectorComplex3D& v);

    /**
     * Operator +.
     */
    virtual VectorComplex3D operator+(const VectorComplex3D& rhs);

    /**
     * Operator -.
     */
    virtual VectorComplex3D operator-(const VectorComplex3D& rhs);

    /**
     * Operator * by real number (rhs).
     */
    virtual VectorComplex3D operator*(const double& rhs);

    /**
     * Operator * by real number (lhs).
     */
    friend VectorComplex3D operator*(const double& lhs, VectorComplex3D& rhs);

    /**
     * Operator * by complex number (rhs).
     */
    virtual VectorComplex3D operator*(const std::complex<double>& rhs);

    /**
     * Operator * by complex number (lhs).
     */
    friend VectorComplex3D operator*(const std::complex<double>& lhs,
            VectorComplex3D& rhs);
};

} /* namespace NumA */

#endif /* VECTORCOMPLEX3D_H_ */
