/*
 * VectorComplex4D.h
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#ifndef VECTORCOMPLEX4D_H_
#define VECTORCOMPLEX4D_H_

#include <complex>

#include "VectorComplexD.h"

namespace NumA {

/**
 * @class VectorComplex4D
 *
 * @brief Vector of complex numbers of size 4.
 *
 * This class represents a vector of complex double precision numbers. The size of this vector is 4.
 */
class VectorComplex4D: public VectorComplexD {

public:

    /**
     * Default constructor.
     */
    VectorComplex4D();

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    VectorComplex4D(const VectorComplex4D& v);

    /**
     * Copy constructor.
     * @param v Object to be copied.
     */
    VectorComplex4D(const VectorComplexD& v);

    /**
     * Destructor.
     */
    ~VectorComplex4D();

    /**
     * Operator =.
     */
    virtual void operator=(const VectorComplex4D& v);

    /**
     * Operator +.
     */
    virtual VectorComplex4D operator+(const VectorComplex4D& rhs);

    /**
     * Operator -.
     */
    virtual VectorComplex4D operator-(const VectorComplex4D& rhs);

    /**
     * Operator * by real number (rhs).
     */
    virtual VectorComplex4D operator*(const double& rhs);

    /**
     * Operator * by real number (lhs).
     */
    friend VectorComplex4D operator*(const double& lhs, VectorComplex4D& rhs);

    /**
     * Operator * by complex number (rhs).
     */
    virtual VectorComplex4D operator*(const std::complex<double>& rhs);

    /**
     * Operator * by complex number (lhs).
     */
    friend VectorComplex4D operator*(const std::complex<double>& lhs,
            VectorComplex4D& rhs);
};

} /* namespace NumA */

#endif /* VECTORCOMPLEX4D_H_ */
