/*
 * NeuralNetworkTypeRegistry.h
 *
 *  Created on: Apr 23, 2016
 *      Author: Pawel Sznajder
 */

#ifndef NEURAL_NETWORK_TYPE_REGISTRY_H_
#define NEURAL_NETWORK_TYPE_REGISTRY_H_

#include <map>

#include "activation_function/ActivationFunction.h"
#include "beans/ActivationFunctionType.h"
#include "beans/CombinationFunctionType.h"
#include "beans/ScalingFunctionType.h"
#include "beans/TrainingFunctionType.h"
#include "combination_function/CombinationFunction.h"
#include "scaling_function/ScalingFunction.h"
#include "training_function/TrainingFunction.h"

namespace NumA {

class NeuralNetworkTypeRegistry {

public:

    virtual ~NeuralNetworkTypeRegistry();

    static NeuralNetworkTypeRegistry* getInstance();

    unsigned int registerActivationFunction(
            const ActivationFunctionType::Type type,
            ActivationFunction* const object);
    unsigned int registerCombinationFunction(
            const CombinationFunctionType::Type type,
            CombinationFunction* const object);
    unsigned int registerScalingFunction(const ScalingFunctionType::Type type,
            ScalingFunction* const object);
    unsigned int registerTrainingFunction(const TrainingFunctionType::Type type,
            TrainingFunction* const object);

    ActivationFunction* getActivationFunction(
            const ActivationFunctionType::Type type) const;
    CombinationFunction* getCombinationFunction(
            const CombinationFunctionType::Type type) const;
    ScalingFunction* getScalingFunction(
            const ScalingFunctionType::Type type) const;
    TrainingFunction* getTrainingFunction(
            const TrainingFunctionType::Type type) const;

private:

    static NeuralNetworkTypeRegistry* p_instance;

    NeuralNetworkTypeRegistry();

    std::map<ActivationFunctionType::Type, ActivationFunction*> m_activationFunctions;
    std::map<CombinationFunctionType::Type, CombinationFunction*> m_combinationFunctions;
    std::map<ScalingFunctionType::Type, ScalingFunction*> m_scalingFunctions;
    std::map<TrainingFunctionType::Type, TrainingFunction*> m_trainingFunctions;
};

}

#endif /* NEURAL_NETWORK_TYPE_REGISTRY_H_ */
