/*
 * RandomGenerator.h
 *
 *  Created on: Aug 1, 2016
 *      Author: Pawel Sznajder
 */

#ifndef RANDOMGENERATOR_H_
#define RANDOMGENERATOR_H_

#include <ElementaryUtils/parameters/Parameters.h>
#include <stddef.h>
#include <string>

namespace NumA {

class RandomGenerator {

public:

    static const std::string PARAM_NAME_SEED; ///< Seed.

    /**
     * Default constructor.
     */
    RandomGenerator();

    /**
     * Destructor
     */
    virtual ~RandomGenerator();

    /**
     * Provides a generic method to configure all types of integrations by passing a Parameters object.
     * Parameters class represents a list of couples key/value (see Parameters class documentation for more info).
     *
     * @param parameters ElemUtils::Parameters object.
     */
    void configure(const ElemUtils::Parameters &parameters);

    /**
     * Set new seed
     * @param seed '0' for time-based seed or fixed seed.
     * @return Value of new seed.
     */
    size_t setSeed(size_t seed);

    /**
     * Get seed
     */
    size_t getSeed() const;

    /**
     * Dice double precision number from flat distribution in range [0, 1).
     */
    double diceFlat() const;

    /**
     * Dice double precision number from flat distribution in given range.
     * @param min Min.
     * @param max Max..
     */
    double diceFlat(double min, double max) const;

    /**
     * Dice double precision number from normal distribution with mean = 0 and sigma = 1.
     */
    double diceNormal() const;

    /**
     * Dice double precision number from normal distribution with given mean and sigma.
     * @param mean Mean.
     * @param sigma Sigma.
     */
    double diceNormal(double mean, double sigma) const;

    /**
     * Dice double precision number from exponential distribution with lambda (negative slope) = 1.
     */
    double diceExp() const;

    /**
     * Dice double precision number from exponential distribution with given lambda (negative slope).
     */
    double diceExp(double lambda) const;

    /**
     * Dice double precision number from exponential distribution with given lambda (negative slope) in given range.
     */
    double diceExp(double min, double max, double lambda) const;

private:

    /**
     * Seed.
     */
    size_t m_seed;
};

}

#endif /* RANDOMGENERATOR_H_ */
