#include <ElementaryUtils/logger/LoggerManager.h>
#include <stddef.h>
#include <cmath>
//#include <stdexcept>
#include <vector>

#define MIN(a, b) ((a) < (b) ? (a) : (b))

namespace NumA {

//TODO refactoring code from python library ; more explicit variable and documentation ; add status return if max iteration reached

class Brent {
public:

    template<class OBJ, typename FUNC>
    double solve(OBJ* object, FUNC function,
            std::vector<double> &funcParameters, double a, double b,
            const double absTolerance = 1e-8, const double relTolerance = 1e-10,
            const size_t maxIteration = 50) {
        double xpre = a, xcur = b;
        double xblk = 0., fpre = 0., fcur = 0., fblk = 0., spre = 0., scur = 0.,
                sbis = 0., tol = 0.;
        double stry = 0., dpre = 0., dblk = 0.;

        fpre = ((*object).*function)(xpre, funcParameters);
        fcur = ((*object).*function)(xcur, funcParameters);

        if ((fpre * fcur) > 0.) {
            throw ElemUtils::CustomException("Brent", __func__,
                    "The root is not bracketed ; verify your interval [a,b]");
        }
        if (fpre == 0.) {
            return xpre;
        }
        if (fcur == 0.) {
            return xcur;
        }

        for (size_t i = 0; i < maxIteration; i++) {
            if ((fpre * fcur) < 0.) {
                xblk = xpre;
                fblk = fpre;
                spre = scur = xcur - xpre;
            }
            if (fabs(fblk) < fabs(fcur)) {
                xpre = xcur;
                xcur = xblk;
                xblk = xpre;

                fpre = fcur;
                fcur = fblk;
                fblk = fpre;
            }

            tol = absTolerance + relTolerance * fabs(xcur);
            sbis = (xblk - xcur) / 2;
            if (fcur == 0 || fabs(sbis) < tol) {
                return xcur;
            }

            if (fabs(spre) > tol && fabs(fcur) < fabs(fpre)) {
                if (xpre == xblk) {
                    /* interpolate */
                    stry = -fcur * (xcur - xpre) / (fcur - fpre);
                } else {
                    /* extrapolate */
                    dpre = (fpre - fcur) / (xpre - xcur);
                    dblk = (fblk - fcur) / (xblk - xcur);
                    stry = -fcur * (fblk * dblk - fpre * dpre)
                            / (dblk * dpre * (fblk - fpre));
                }
                if (2 * fabs(stry) < MIN(fabs(spre), 3 * fabs(sbis) - tol)) {
                    /* good short step */
                    spre = scur;
                    scur = stry;
                } else {
                    /* bisect */
                    spre = sbis;
                    scur = sbis;
                }
            } else {
                /* bisect */
                spre = sbis;
                scur = sbis;
            }

            xpre = xcur;
            fpre = fcur;
            if (fabs(scur) > tol) {
                xcur += scur;
            } else {
                xcur += (sbis > 0 ? tol : -tol);
            }

            fcur = ((*object).*function)(xcur, funcParameters);
        }

        return xcur;
    }
};

} // namespace NumA
