#ifndef DIFFERENCES_H
#define DIFFERENCES_H

/**
 * @file Differences.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @date 10 December 2015
 * @version 1.0
 */

#include <string>

namespace NumA {


/**
 * @class Differences
 *
 * @brief Class defining absolute and relative differences for comparison of real numbers (double).
 */
class Differences {
public:
    /**
     * Default constructor (differences equal to 0).
     */
    Differences();
    /**
     * Constructor.
     * @param absoluteDifference Absolute difference.
     * @param relativeDifference Relative difference.
     */
    Differences(const double absoluteDifference,
            const double relativeDifference);
    /**
     * Default destructor.
     */
    virtual ~Differences();

    /**
     *
     * @return Pre-formatted string with the differences.
     */
    std::string toString() const;

    /**
     *
     * @return Absolute difference.
     */
    double getAbsoluteDifference() const;
    /**
     *
     * @return Relative difference.
     */
    double getRelativeDifference() const;

    /**
     *
     * @param absoluteDifference Absolute difference.
     */
    void setAbsoluteDifference(double absoluteDifference);
    /**
     *
     * @param relativeDifference Relative difference.
     */
    void setRelativeDifference(double relativeDifference);

private:
    double m_absoluteDifference; ///< Absolute difference.
    double m_relativeDifference; ///< Relative difference.
};

} // namespace NumA

#endif /* DIFFERENCES_H */
