#ifndef ERRORS_H
#define ERRORS_H

/**
 * @file Errors.h
 * @author Bryan BERTHOU (SPhN / CEA Saclay)
 * @date 21 March 2016
 * @version 1.0
 */

namespace NumA {

/**
 * @class Errors
 *
 * @brief Class for defining estimations of absolute and relative errors.
 */
class Errors {
public:
    /**
     * Default constructor (errors equal to 0).
     */
    Errors();
    /**
     * Constructor.
     * @param absolute Absolute error.
     * @param relative Relative error.
     */
    Errors(const double absolute, const double relative);
    /**
     * Default destructor.
     */
    virtual ~Errors();

    // ##### GETTERS & SETTERS #####

    /**
     *
     * @return Absolute error.
     */
    double getAbsolute() const;
    /**
     *
     * @param absolute Absolute error.
     */
    void setAbsolute(double absolute);
    /**
     *
     * @return Relative error.
     */
    double getRelative() const;
    /**
     *
     * @param relative Relative error.
     */
    void setRelative(double relative);

private:
    double m_absolute; ///< Absolute error.
    double m_relative; ///< Relative error.
};

} /* namespace NumA */

#endif /* ERRORS_H */
