#ifndef TOLERANCES_H_
#define TOLERANCES_H_

/**
 * @file Tolerances.h
 * @author: Hervé MOUTARDE (SPhN / CEA Saclay)
 * @date 3 July 2015
 * @version 1.0
 */

#include <string>

namespace NumA {

/**
 * @class Tolerances
 *
 * @brief Define absolute and relative tolerances for comparison of real numbers (double) and check if they are positive.
 */
class Tolerances {
public:
    /**
     * Default constructor (tolerances equal to 0).
     */
    Tolerances();
    /**
     * Constructor.
     * @param absoluteTolerance Absolute tolerance.
     * @param relativeTolerance Relative tolerance.
     */
    Tolerances(double absoluteTolerance, double relativeTolerance);
    /**
     * Default destructor.
     */
    virtual ~Tolerances();

    /**
     *
     * @return Pre-formatted string with the tolerances.
     */
    std::string toString() const;

    // ##### GETTERS & SETTERS #####

    /**
     *
     * @return Absolute tolerance.
     */
    double getAbsoluteTolerance() const;
    /**
     *
     * @return Relative tolerance.
     */
    double getRelativeTolerance() const;

    /**
     *
     * @param absoluteTolerance Absolute tolerance.
     */
    void setAbsoluteTolerance(double absoluteTolerance);
    /**
     *
     * @param relativeTolerance Relative tolerance.
     */
    void setRelativeTolerance(double relativeTolerance);

private:
    double m_absoluteTolerance; ///< Absolute tolerance.
    double m_relativeTolerance; ///< Relative tolerance.

    void testPositivity(const double &tolerance) const; ///< Positivity test.
};

} // namespace NumA

#endif /* TOLERANCES_H_ */
