#include "../../../../include/NumA/integration/one_dimension/ChebyshevAIntegrator1D.h"

#include <vector>
#include <ElementaryUtils/logger/CustomException.h>
#include <math.h>

namespace NumA {

ChebyshevAIntegrator1D::ChebyshevAIntegrator1D(unsigned int N) :
        QuadratureIntegrator1D(N) {
    setNodesAndWeights();
}

ChebyshevAIntegrator1D::ChebyshevAIntegrator1D(
        const ChebyshevAIntegrator1D &other) :
        QuadratureIntegrator1D(other) {
}

ChebyshevAIntegrator1D::~ChebyshevAIntegrator1D() {
}

ChebyshevAIntegrator1D* ChebyshevAIntegrator1D::clone() const {
    return new ChebyshevAIntegrator1D(*this);
}

void ChebyshevAIntegrator1D::setN(unsigned int n) {
    if (n != m_N) {
        m_N = n;
        setNodesAndWeights();
    }
}

void ChebyshevAIntegrator1D::setNodesAndWeights() {

    double phi = 0.;

    //clear nodes and weights
    m_nodes.clear();
    m_weights.clear();

    // Check number of steps
    if (m_N < 1) {
        throw ElemUtils::CustomException("ChebyshevAIntegrator1D", __func__,
                "Invalid number of nodes. Must be > 0");
    } else {
        m_nodes.assign(m_N, 0.);
        m_weights.assign(m_N, 0.);

        for (unsigned int i = 0; i < m_N; i++) {
            phi = M_PI * double(m_N - i) / double(m_N + 1);
            m_nodes.at(i) = cos(phi);
            m_weights.at(i) = M_PI / double(m_N + 1) * (sin(phi) * sin(phi));
        }
    }
}

} /* namespace NumA */
