#include "../../../../include/NumA/integration/one_dimension/ChebyshevBIntegrator1D.h"

#include <vector>
#include <ElementaryUtils/logger/CustomException.h>
#include <math.h>

namespace NumA {

ChebyshevBIntegrator1D::ChebyshevBIntegrator1D(unsigned int N) :
        QuadratureIntegrator1D(N) {
    setNodesAndWeights();
}

ChebyshevBIntegrator1D::ChebyshevBIntegrator1D(
        const ChebyshevBIntegrator1D &other) :
        QuadratureIntegrator1D(other) {
}

ChebyshevBIntegrator1D::~ChebyshevBIntegrator1D() {
}

ChebyshevBIntegrator1D* ChebyshevBIntegrator1D::clone() const {
    return new ChebyshevBIntegrator1D(*this);
}

void ChebyshevBIntegrator1D::setN(unsigned int n) {
    if (n != m_N) {
        m_N = n;
        setNodesAndWeights();
    }
}

void ChebyshevBIntegrator1D::setNodesAndWeights() {

    double phi = 0.;

    //clear nodes and weights
    m_nodes.clear();
    m_weights.clear();

    // Check number of steps
    if (m_N < 1) {
        throw ElemUtils::CustomException("ChebyshevBIntegrator1D", __func__,
                "Invalid number of nodes. Must be > 0");
    } else {
        m_nodes.assign(m_N, 0.);
        m_weights.assign(m_N, 0.);

        for (unsigned int i = 0; i < m_N; i++) {
            phi = M_PI * double(2 * m_N - 1 - 2 * i) / double(2 * m_N);
            m_nodes.at(i) = cos(phi);
            m_weights.at(i) = M_PI / double(m_N);
        }
    }
}

} /* namespace NumA */
