#include "../../../../include/NumA/integration/one_dimension/GaussLegendreIntegrator1D.h"

#include <vector>
#include <ElementaryUtils/logger/CustomException.h>
#include <math.h>

//#include "../../../../include/NumA/utils/GenericType.h"
//#include "../../../../include/NumA/utils/Parameters.h"

namespace NumA {

GaussLegendreIntegrator1D::GaussLegendreIntegrator1D(unsigned int N) :
        QuadratureIntegrator1D(N) {
    setNodesAndWeights();
}

GaussLegendreIntegrator1D::GaussLegendreIntegrator1D(
        const GaussLegendreIntegrator1D &other) :
        QuadratureIntegrator1D(other) {
}

GaussLegendreIntegrator1D::~GaussLegendreIntegrator1D() {
    // TODO Auto-generated destructor stub
}

GaussLegendreIntegrator1D* GaussLegendreIntegrator1D::clone() const {
    return new GaussLegendreIntegrator1D(*this);
}

void GaussLegendreIntegrator1D::setN(unsigned int n) {
    if (n != m_N) {
        m_N = n;
        setNodesAndWeights();
    }
}

void GaussLegendreIntegrator1D::setNodesAndWeights() {
    //clear nodes and weights
    m_nodes.clear();
    m_weights.clear();

    // Check number of steps
    if (m_N <= 0) {
        throw ElemUtils::CustomException("GaussLegendreIntegrator1D", __func__,
                "Invalid number of nodes. Must be > 0");
    } else {
        m_nodes.assign(m_N, 0.);
        m_weights.assign(m_N, 0.);
        int m1 = m_N / 2;
        int m2 = (m_N + 1) / 2;

        if (m_N == 20) {

            // Defining the absolute value of the nodes of GL20P
            m_nodes[m1] = 0.0765265211334973;
            m_nodes[m1 + 1] = 0.2277858511416451;
            m_nodes[m1 + 2] = 0.3737060887154195;
            m_nodes[m1 + 3] = 0.5108670019508271;
            m_nodes[m1 + 4] = 0.6360536807265150;
            m_nodes[m1 + 5] = 0.7463319064601508;
            m_nodes[m1 + 6] = 0.8391169718222188;
            m_nodes[m1 + 7] = 0.9122344282513259;
            m_nodes[m1 + 8] = 0.9639719272779138;
            m_nodes[m1 + 9] = 0.9931285991850949;

            // Defining the weights associated with the previous nodes
            m_weights[m1] = 0.1527533871307258;
            m_weights[m1 + 1] = 0.1491729864726037;
            m_weights[m1 + 2] = 0.1420961093183820;
            m_weights[m1 + 3] = 0.1316886384491766;
            m_weights[m1 + 4] = 0.1181945319615184;
            m_weights[m1 + 5] = 0.1019301198172404;
            m_weights[m1 + 6] = 0.0832767415767048;
            m_weights[m1 + 7] = 0.0626720483341091;
            m_weights[m1 + 8] = 0.0406014298003869;
            m_weights[m1 + 9] = 0.0176140071391521;

        } else if (m_N == 40) {

            // Defining the absolute value of the nodes of GL40P
            m_nodes[m1] = 0.0387724175060508;
            m_nodes[m1 + 1] = 0.1160840706752552;
            m_nodes[m1 + 2] = 0.1926975807013711;
            m_nodes[m1 + 3] = 0.2681521850072537;
            m_nodes[m1 + 4] = 0.3419940908257585;
            m_nodes[m1 + 5] = 0.4137792043716050;
            m_nodes[m1 + 6] = 0.4830758016861787;
            m_nodes[m1 + 7] = 0.5494671250951282;
            m_nodes[m1 + 8] = 0.6125538896679802;
            m_nodes[m1 + 9] = 0.6719566846141796;
            m_nodes[m1 + 10] = 0.7273182551899271;
            m_nodes[m1 + 11] = 0.7783056514265194;
            m_nodes[m1 + 12] = 0.8246122308333117;
            m_nodes[m1 + 13] = 0.8659595032122595;
            m_nodes[m1 + 14] = 0.9020988069688743;
            m_nodes[m1 + 15] = 0.9328128082786765;
            m_nodes[m1 + 16] = 0.9579168192137917;
            m_nodes[m1 + 17] = 0.9772599499837743;
            m_nodes[m1 + 18] = 0.9907262386994570;
            m_nodes[m1 + 19] = 0.9982377097105593;

            // Defining the weights associated with the previous nodes
            m_weights[m1] = 0.0775059479784248;
            m_weights[m1 + 1] = 0.0770398181642480;
            m_weights[m1 + 2] = 0.0761103619006262;
            m_weights[m1 + 3] = 0.0747231690579683;
            m_weights[m1 + 4] = 0.0728865823958041;
            m_weights[m1 + 5] = 0.0706116473912868;
            m_weights[m1 + 6] = 0.0679120458152339;
            m_weights[m1 + 7] = 0.0648040134566010;
            m_weights[m1 + 8] = 0.0613062424929289;
            m_weights[m1 + 9] = 0.0574397690993916;
            m_weights[m1 + 10] = 0.0532278469839368;
            m_weights[m1 + 11] = 0.0486958076350722;
            m_weights[m1 + 12] = 0.0438709081856733;
            m_weights[m1 + 13] = 0.0387821679744720;
            m_weights[m1 + 14] = 0.0334601952825478;
            m_weights[m1 + 15] = 0.0279370069800234;
            m_weights[m1 + 16] = 0.0222458491941670;
            m_weights[m1 + 17] = 0.0164210583819079;
            m_weights[m1 + 18] = 0.0104982845311528;
            m_weights[m1 + 19] = 0.0045212770985332;

        } else { // Compute roots and weights if not prestored

            //variables
            double z1, z, pp, p3, p2, p1;

            const double EPS = 1.E-12;

            //loop over desired roots
            for (unsigned int i = 1; i <= m2; i++) {

                //starting approximation to ith root
                z = cos(M_PI * (i - 0.25) / (m_N + 0.5));

                do {
                    p1 = 1.0;
                    p2 = 0.0;

                    for (unsigned int j = 1; j <= m_N; j++) {
                        p3 = p2;
                        p2 = p1;
                        p1 = ((2. * j - 1.) * z * p2 - (j - 1.) * p3) / j;
                    }

                    pp = m_N * (z * p1 - p2) / (z * z - 1.);
                    z1 = z;

                    //Newton-Raphson interpolation formula
                    z = z1 - p1 / pp;

                } while (fabs(z - z1) > EPS);

                //save results
                m_nodes[m_N - i] = z;
                m_weights[m_N - i] = 2. / ((1.0 - z * z) * pp * pp);
            }
        }

        // Complete with the negative nodes
        for (unsigned int i = 0; i < m1; i++) {
            m_nodes[i] = -m_nodes[m_N - i - 1];
            m_weights[i] = m_weights[m_N - i - 1];
        }
    }
}

} /* namespace NumA */
