#include "../../../../include/NumA/integration/one_dimension/GaussLegendreSeStIntegrator1D.h"

#include <algorithm>
#include <cmath>
#include <limits>
//#include <stdexcept>
#include <vector>
#include <ElementaryUtils/logger/CustomException.h>

//#include "../../../../include/NumA/utils/GenericType.h"
//#include "../../../../include/NumA/utils/Parameters.h"

namespace NumA {

GaussLegendreSeStIntegrator1D::GaussLegendreSeStIntegrator1D(unsigned int N) :
        QuadratureIntegrator1D(N) {
    setNodesAndWeights();
}

GaussLegendreSeStIntegrator1D::GaussLegendreSeStIntegrator1D(
        const GaussLegendreSeStIntegrator1D &other) :
        QuadratureIntegrator1D(other) {
}

GaussLegendreSeStIntegrator1D::~GaussLegendreSeStIntegrator1D() {
    // TODO Auto-generated destructor stub
}

GaussLegendreSeStIntegrator1D* GaussLegendreSeStIntegrator1D::clone() const {
    return new GaussLegendreSeStIntegrator1D(*this);
}

void GaussLegendreSeStIntegrator1D::setN(unsigned int n) {
    if (n != m_N) {
        m_N = n;
        setNodesAndWeights();
    }
}

void GaussLegendreSeStIntegrator1D::setNodesAndWeights() {
    //clear nodes and weights
    m_nodes.clear();
    m_weights.clear();

    // Check number of steps
    if (m_N <= 0) {
        throw ElemUtils::CustomException("GaussLegendreSeStIntegrator1D",
                __func__, "Invalid number of nodes. Must be > 0");
    } else {
        m_nodes.assign(m_N, 0.);
        m_weights.assign(m_N, 0.);

        std::vector<double> c(m_N, 0.);
        double cc = 1.;
        double r = 0.;
        double xtemp = 0.;
        unsigned int const maxit = 10;
        double p0, dp0, p1, dp1, p2, dp2, d;

        for (unsigned int i = 0; i < m_N; i++) {
            c[i] = double(i * i) / double((2 * i + 1) * (2 * i - 1));
        }

        for (unsigned int i = 1; i < m_N; i++) {
            cc *= c[i];
        }
        cc = 2 * cc;

        for (unsigned int i = 0; i < m_N; i++) {
            if (i == 0) {
                r = 2.78 / (4.0 + double(m_N * m_N));
                xtemp = 1.0 - r;
            } else if (i == 1) {
                r = 1.0 + 0.06 * ((double) m_N - 8) / ((double) m_N);
                xtemp = xtemp - 4.1 * r * (1.0 - xtemp);
            } else if (i == 2) {
                r = 1.0 + 0.22 * ((double) m_N - 8) / ((double) m_N);
                xtemp = xtemp - 1.67 * r * (m_nodes[0] - xtemp);
            } else if (i < m_N - 2) {
                xtemp = 3.0 * m_nodes[i - 1] - 3.0 * m_nodes[i - 2]
                        + m_nodes[i - 3];
            } else if (i == m_N - 2) {
                r = 1.0
                        / (1.0
                                + 0.639 * ((double) m_N - 4)
                                        / (1.0 + 0.71 * ((double) m_N - 4)));
                xtemp = xtemp + r * (xtemp - m_nodes[i - 2]) / 0.766;
            } else if (i == m_N - 1) {
                r = 1.0 / (1.0 + 0.22 * ((double) m_N - 8) / ((double) m_N));
                xtemp = xtemp + r * (xtemp - m_nodes[i - 2]) / 1.67;
            }

            for (unsigned int j = 1; j <= maxit; j++) {
                p1 = 1.;
                dp1 = 0.;
                p2 = xtemp;
                dp2 = 1.;
                for (unsigned int k = 1; k < m_N; k++) {
                    p0 = p1;
                    dp0 = dp1;

                    p1 = p2;
                    dp1 = dp2;

                    p2 = xtemp * p1 - c[k] * p0;
                    dp2 = xtemp * dp1 + p1 - c[k] * dp0;
                }

                d = p2 / dp2;
                xtemp = xtemp - d;
                if (std::abs(d)
                        < std::numeric_limits<double>::epsilon()
                                * (std::abs(d) + 1.))
                    break;
            }

            m_nodes[i] = xtemp;
            m_weights[i] = cc / dp2 / p1;
        }
        std::reverse(m_nodes.begin(), m_nodes.end());
        std::reverse(m_weights.begin(), m_weights.end());
    }
}

} /* namespace NumA */
