#include "../../../../include/NumA/integration/one_dimension/Integrator1D.h"

#include <ElementaryUtils/logger/CustomException.h>
#include <ElementaryUtils/parameters/GenericType.h>
#include <ElementaryUtils/parameters/Parameters.h>
#include <string>

#include "../../../../include/NumA/integration/one_dimension/DExpIntegrator1D.h"
#include "../../../../include/NumA/integration/one_dimension/gauss_kronrod_adaptive/GaussKronrodAdaptive.h"
#include "../../../../include/NumA/integration/one_dimension/GaussLegendreIntegrator1D.h"
#include "../../../../include/NumA/integration/one_dimension/IntegratorType1D.h"
#include "../../../../include/NumA/integration/one_dimension/TrapezoidalIntegrator1D.h"
#include "../../../../include/NumA/integration/one_dimension/TrapezoidalLogIntegrator1D.h"
#include "../../../../include/NumA/utils/Errors.h"
#include "../../../../include/NumA/utils/Tolerances.h"

namespace NumA {

const std::string Integrator1D::PARAM_NAME_ABSOLUTE_TOLERANCE =
        "absoluteTolerance";
const std::string Integrator1D::PARAM_NAME_RELATIVE_TOLERANCE =
        "relativeTolerance";

Integrator1D::Integrator1D() :
        m_tolerances(Tolerances(1.e-3, 1.e-3)) {
}

Integrator1D::Integrator1D(const Integrator1D &other) {
    m_tolerances = other.m_tolerances;
}

Integrator1D::~Integrator1D() {
}

Integrator1D* Integrator1D::newIntegrator(
        const IntegratorType1D::Type &oneDimIntegratorType) {
    switch (oneDimIntegratorType) {
    case IntegratorType1D::TRAPEZOIDAL:
        return new TrapezoidalIntegrator1D();
        break;
    case IntegratorType1D::TRAPEZOIDALLOG:
        return new TrapezoidalLogIntegrator1D();
        break;
    case IntegratorType1D::GL:
        return new GaussLegendreIntegrator1D();
        break;
    case IntegratorType1D::DEXP:
        return new DExpIntegrator1D();
        break;
    case IntegratorType1D::GK21_ADAPTIVE:
        return new GaussKronrodAdaptive();
        break;
    default:
        throw ElemUtils::CustomException("Integrator1D", __func__,
                "Unknown or undefined integrator.");
        break;
    }

    return 0;
}

const Tolerances& NumA::Integrator1D::getTolerances() const {
    return m_tolerances;
}

void NumA::Integrator1D::setTolerances(const Tolerances& tolerances) {
    m_tolerances = tolerances;
}

void NumA::Integrator1D::configure(const ElemUtils::Parameters& parameters) {
    if (parameters.isAvailable(Integrator1D::PARAM_NAME_ABSOLUTE_TOLERANCE)) {
        m_tolerances.setAbsoluteTolerance(
                parameters.getLastAvailable().toDouble());
    } else if (parameters.isAvailable(
            Integrator1D::PARAM_NAME_RELATIVE_TOLERANCE)) {
        m_tolerances.setRelativeTolerance(
                parameters.getLastAvailable().toDouble());
    }
}

const Errors& NumA::Integrator1D::getErrors() const {
    return m_errors;
}

void NumA::Integrator1D::setErrors(const Errors& errors) {
    m_errors = errors;
}

} // namespace NumA
