#include "../../../../include/NumA/integration/one_dimension/IntegratorType1D.h"

#include <ElementaryUtils/logger/LoggerManager.h>
#include <ElementaryUtils/string_utils/Formatter.h>
#include <ElementaryUtils/string_utils/StringUtils.h>

namespace NumA {

IntegratorType1D::IntegratorType1D() :
		m_type(IntegratorType1D::UNDEFINED) {
}

IntegratorType1D::IntegratorType1D(Type type) :
		m_type(type) {
}

IntegratorType1D::IntegratorType1D(const std::string& integratorType1DString) {

	if (ElemUtils::StringUtils::equals(integratorType1DString, "TRAPEZOIDAL")) {
		m_type = IntegratorType1D::TRAPEZOIDAL;
	} else if (ElemUtils::StringUtils::equals(integratorType1DString,
			"TRAPEZOIDALLOG")) {
		m_type = IntegratorType1D::TRAPEZOIDALLOG;
	} else if (ElemUtils::StringUtils::equals(integratorType1DString,
			"GL")) {
		m_type = IntegratorType1D::GL;
	} else if (ElemUtils::StringUtils::equals(integratorType1DString, "DEXP")) {
		m_type = IntegratorType1D::DEXP;
	} else if (ElemUtils::StringUtils::equals(integratorType1DString,
			"GK21_ADAPTIVE")) {
		m_type = IntegratorType1D::GK21_ADAPTIVE;
	} else {
		ElemUtils::LoggerManager::getInstance()->warn("IntegratorType1D", __func__,
				ElemUtils::Formatter()
						<< "Cannot found IntegratorType1D for string = "
						<< integratorType1DString);
	}
}

IntegratorType1D::~IntegratorType1D() {
}

IntegratorType1D::operator IntegratorType1D::Type() const {
	return m_type;
}

std::string IntegratorType1D::toString() const {
	switch (m_type) {
	case TRAPEZOIDAL:
		return "TRAPEZOIDAL";
		break;
	case TRAPEZOIDALLOG:
		return "TRAPEZOIDALLOG";
		break;
	case GL:
		return "GL";
		break;
	case DEXP:
		return "DEXP";
		break;
	case GK21_ADAPTIVE:
		return "GK21_ADAPTIVE";
		break;
	default:
		return "UNDEFINED";
	}
}

std::string IntegratorType1D::getShortName() {
	switch (m_type) {
	case TRAPEZOIDAL:
		return "TRAPEZOIDAL";
		break;
	case TRAPEZOIDALLOG:
		return "TRAPEZOIDALLOG";
		break;
	case GL:
		return "GL";
		break;
	case DEXP:
		return "DEXP";
		break;
	case GK21_ADAPTIVE:
		return "GK21_ADAPTIVE";
		break;
	default:
		return "UNDEFINED";
	}
}

IntegratorType1D::Type IntegratorType1D::getType() const {
	return m_type;
}

void IntegratorType1D::setType(Type type) {
	m_type = type;
}

} /* namespace NumA */
