#include "../../../../include/NumA/integration/one_dimension/QuadratureIntegrator1D.h"

#include <string>
#include <vector>
#include <ElementaryUtils/parameters/GenericType.h>
#include <ElementaryUtils/parameters/Parameters.h>

#include "../../../../include/NumA/functor/one_dimension/FunctionType1D.h"

namespace NumA {

const std::string QuadratureIntegrator1D::PARAM_NAME_N = "numberOfNodes";

QuadratureIntegrator1D::QuadratureIntegrator1D(unsigned int N) :
        Integrator1D(), m_N(N) {
}

QuadratureIntegrator1D::QuadratureIntegrator1D(
        const QuadratureIntegrator1D &other) :
        Integrator1D(other) {
    m_nodes = other.m_nodes;
    m_weights = other.m_weights;
    m_N = other.m_N;
}

QuadratureIntegrator1D::~QuadratureIntegrator1D() {
}

double QuadratureIntegrator1D::integrate(FunctionType1D* pFunction, double a,
        double b, std::vector<double>& parameters) {
    double result = 0., x = 0.;

    // Shifting integration interval to [-1;1]
    double c, d;
    d = (a + b) * 0.5;
    c = (b - a) * 0.5;

    // Entering the loop
    for (unsigned int i = 0; i < m_nodes.size(); i++) {
        x = d + c * m_nodes[i];
        result += ((*pFunction)(x, parameters)) * m_weights[i];
    }
    // Multiplying by the step at the end to save computing time
    result = result * c;

    return result;
}

void QuadratureIntegrator1D::configure(const ElemUtils::Parameters &parameters) {
    Integrator1D::configure(parameters);

    if (parameters.isAvailable(QuadratureIntegrator1D::PARAM_NAME_N)) {
        setN(parameters.getLastAvailable().toUInt());
    }
}

unsigned int QuadratureIntegrator1D::getN() const {
    return m_N;
}

const std::vector<double>& QuadratureIntegrator1D::getNodes() const {
    return m_nodes;
}

const std::vector<double>& QuadratureIntegrator1D::getWeights() const {
    return m_weights;
}

} // namespace NumA
