#include "../../../../include/NumA/integration/one_dimension/TrapezoidalIntegrator1D.h"

#include <vector>

namespace NumA {

TrapezoidalIntegrator1D::TrapezoidalIntegrator1D(unsigned int N) :
        QuadratureIntegrator1D(N) {
    setNodesAndWeights();
}

TrapezoidalIntegrator1D::TrapezoidalIntegrator1D(
        const TrapezoidalIntegrator1D &other) :
        QuadratureIntegrator1D(other) {
}

TrapezoidalIntegrator1D::~TrapezoidalIntegrator1D() {
}

TrapezoidalIntegrator1D* TrapezoidalIntegrator1D::clone() const {
    return new TrapezoidalIntegrator1D(*this);
}

void TrapezoidalIntegrator1D::setN(unsigned int n) {
    if (n != m_N) {
        m_N = n;
        setNodesAndWeights();
    }
}

void TrapezoidalIntegrator1D::setNodesAndWeights() {
    double step = 2. / (m_N - 1.);
    double x = -1.;

    m_nodes.assign(m_N, x);
    m_nodes.at(m_N - 1) = 1.;
    m_weights.assign(m_N, step);
    m_weights.at(0) *= 0.5;
    m_weights.at(m_N - 1) *= 0.5;
    // Entering the loop
    for (unsigned int i = 1; i < m_N - 1; i++) {
        x += step;
        m_nodes.at(i) = x;
    }
}

} /* namespace NumA */
