#include "../../../../include/NumA/linear_algebra/matrix/MatrixD.h"

#include <cstdarg>
#include <iterator>
#include <ElementaryUtils/logger/CustomException.h>
#include <ElementaryUtils/string_utils/Formatter.h>

namespace NumA {

MatrixD::MatrixD() :
        m_rowsNumber(0), m_columnsNumber(0) {
}

MatrixD::MatrixD(const size_t _rowsNumber, const size_t _columnsNumber) :
        m_rowsNumber(_rowsNumber), m_columnsNumber(_columnsNumber) {
    m_matrix.assign(m_rowsNumber * m_columnsNumber, 0.);
}

// TODO Write something better?
// If the number and type of arguments are not right, UNDEFINED BEHAVIOUR !!!
// This constructor is very DANGEROUS!!! USE IT RIGHT, OR SCRAP IT ALL AND REWRITE SOMETHING BETTER!
MatrixD::MatrixD(const size_t _rowsNumber, const size_t _columnsNumber,
        double first_value, ...) :
        m_rowsNumber(_rowsNumber), m_columnsNumber(_columnsNumber) {

    va_list ap;
    va_start(ap, first_value);

    m_matrix.assign(m_rowsNumber * m_columnsNumber, 0.);
    m_matrix.at(0) = first_value;
    for (unsigned int i = 1; i < m_matrix.size(); i++) {
        m_matrix.at(i) = va_arg(ap, double);
    }
    va_end(ap);
}

MatrixD::MatrixD(const MatrixD &other) {
    m_matrix = other.m_matrix;
    m_rowsNumber = other.m_rowsNumber;
    m_columnsNumber = other.m_columnsNumber;
}

MatrixD::MatrixD(const VectorD& rhs) {
    m_matrix = rhs.toStdVector();
    m_rowsNumber = rhs.size();
    m_columnsNumber = 1;
}

MatrixD::~MatrixD() {
}

NumA::VectorD MatrixD::operator*(const NumA::VectorD& rhs) const {

// test vector's size and matrix's size
    if (rhs.size() != cols()) {
        ElemUtils::Formatter formatter;
        formatter << "Vector's size = " << rhs.size()
                << " does not match matrix size = " << cols() << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }

    NumA::VectorD result(rows(), 0.0);

    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i) += this->at(i, j) * rhs.at(j);
        }
    }

    return result;
}

//std::vector<double> MatrixD::operator*=(const std::vector<double>& rhs) {
//	std::vector<double> result = (*this) * rhs;
//	return result;
//}

NumA::MatrixD MatrixD::operator *(const NumA::MatrixD& rhs) const {
    // test vector's size and matrix's size
    if (rhs.rows() != cols()) {
        ElemUtils::Formatter formatter;
        formatter << "Right matrix's rows number = " << rhs.rows()
                << " does not match left matrix's columns number = " << cols()
                << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }

    NumA::MatrixD result(rows(), rhs.cols());

    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < rhs.cols(); j++) {
            for (unsigned k = 0; k < cols(); k++) {
                result.at(i, j) += this->at(i, k) * rhs.at(k, j);
            }
        }
    }

    return result;
}

NumA::MatrixD MatrixD::operator +(const NumA::MatrixD& rhs) const {
    // test vector's size and matrix's size
    if (rhs.rows() != rows()) {
        ElemUtils::Formatter formatter;
        formatter << "Right matrix's rows number = " << rhs.rows()
                << " does not match left matrix's rows number = " << rows()
                << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }
    if (rhs.cols() != cols()) {
        ElemUtils::Formatter formatter;
        formatter << "Right matrix's columns number = " << rhs.cols()
                << " does not match left matrix's columns number = " << cols()
                << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }

    NumA::MatrixD result(rows(), cols());

    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i, j) = this->at(i, j) + rhs.at(i, j);
        }
    }

    return result;
}

NumA::MatrixD MatrixD::operator -(const NumA::MatrixD& rhs) const {
    // test vector's size and matrix's size
    if (rhs.rows() != rows()) {
        ElemUtils::Formatter formatter;
        formatter << "Right matrix's rows number = " << rhs.rows()
                << " does not match left matrix's rows number = " << rows()
                << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }
    if (rhs.cols() != cols()) {
        ElemUtils::Formatter formatter;
        formatter << "Right matrix's columns number = " << rhs.cols()
                << " does not match left matrix's columns number = " << cols()
                << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }

    NumA::MatrixD result(rows(), cols());

    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i, j) = this->at(i, j) - rhs.at(i, j);
        }
    }

    return result;
}

NumA::MatrixD MatrixD::operator *(double rhs) const {
    MatrixD result(rows(), cols());
    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i, j) = this->at(i, j) * rhs;
        }
    }
    return result;
}

NumA::MatrixD MatrixD::operator +(double rhs) const {
    MatrixD result(rows(), cols());
    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i, j) = this->at(i, j) + rhs;
        }
    }
    return result;
}

NumA::MatrixD MatrixD::operator -(double rhs) const {
    MatrixD result(rows(), cols());
    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i, j) = this->at(i, j) - rhs;
        }
    }
    return result;
}

NumA::MatrixD MatrixD::operator /(double rhs) const {
    MatrixD result(rows(), cols());
    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            result.at(i, j) = this->at(i, j) / rhs;
        }
    }
    return result;
}

NumA::MatrixD MatrixD::transpose() const {
    MatrixD matTransposed(cols(), rows());
    for (unsigned i = 0; i < rows(); i++) {
        for (unsigned j = 0; j < cols(); j++) {
            matTransposed.at(j, i) = this->at(i, j);
        }
    }
    return matTransposed;
}

NumA::MatrixD MatrixD::Id(unsigned int n) {
    MatrixD identity(n, n);
    for (unsigned int i = 0; i < n; i++) {
        identity.at(i, i) = 1.;
    }
    return identity;
}

void MatrixD::update(const size_t i, const size_t j, const double value) {
    if (i > m_rowsNumber || j > m_columnsNumber) {
        ElemUtils::Formatter formatter;
        formatter << "i or j index does not match matrix size rows = "
                << m_rowsNumber << " cols = " << m_columnsNumber << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }

    at(i, j) = value;
}

std::string MatrixD::toString() const {
    ElemUtils::Formatter formatter;

    for (unsigned int i = 0; i < rows(); i++) {
        for (unsigned int j = 0; j < cols(); j++) {
            formatter << at(i, j);
            formatter << " ";
        }

        formatter << "\n";
    }

    return formatter.str();
}

void MatrixD::setLine(size_t i, const NumA::VectorD& line) {
    if (line.size() != cols()) {
        //TODO print more information
        throw ElemUtils::CustomException("MatrixD", __func__,
                "Line size doesn't match matrix row size.");
    }
    if (i >= rows()) {
        //TODO print more information
        throw ElemUtils::CustomException("MatrixD", __func__,
                "Line index is out of bound!");
    }
    for (unsigned int j = 0; j < line.size(); j++) {
        at(i, j) = line.at(j);
    }
}

void MatrixD::addLine(size_t i, const NumA::VectorD& line) {
    if (line.size() != cols()) {
        //TODO print more information
        throw ElemUtils::CustomException("MatrixD", __func__,
                "Line size doesn't match matrix row size.");
    }
    if (i > rows()) {
        //TODO print more information
        throw ElemUtils::CustomException("MatrixD", __func__,
                "Line index is out of bound!");
    }
    std::vector<double> lineVector = line.toStdVector();
    m_matrix.insert(m_matrix.begin() + i * cols(), lineVector.begin(),
            lineVector.end());
    m_rowsNumber++;
}

void MatrixD::appendLine(const NumA::VectorD& line) {
    if (line.size() != cols()) {
        //TODO print more information
        throw ElemUtils::CustomException("MatrixD", __func__,
                "Line size doesn't match matrix row size.");
    }
    std::vector<double> lineVector = line.toStdVector();
    m_matrix.insert(m_matrix.end(), lineVector.begin(), lineVector.end());
    m_rowsNumber++;
}

NumA::VectorD MatrixD::getLine(const size_t lineIndex) const {
    if (lineIndex >= rows()) {
        //TODO print more information
        throw ElemUtils::CustomException("MatrixD", __func__,
                "Line index is out of bound!");
    }

    VectorD result;

    for (size_t i = cols() * lineIndex; i != cols() * (lineIndex + 1); i++) {
        result.push_back(m_matrix[i]);
    }

    return result;
}

void MatrixD::assign(const size_t _rowsNumber, const size_t _columnsNumber,
        double value) {
    m_matrix.assign(_rowsNumber * _columnsNumber, value);
    m_rowsNumber = _rowsNumber;
    m_columnsNumber = _columnsNumber;
}

size_t MatrixD::cols() const {
    return m_columnsNumber;
}

size_t MatrixD::rows() const {
    return m_rowsNumber;
}

double& MatrixD::at(const size_t i, const size_t j) {
    if (j >= cols()) {
        ElemUtils::Formatter formatter;
        formatter
                << "Matrix column index out of bounds! Must be strictly inferior to "
                << cols() << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }
    if (i >= rows()) {
        ElemUtils::Formatter formatter;
        formatter
                << "Matrix row index out of bounds! Must be strictly inferior to "
                << rows() << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }
    return m_matrix.at(i * cols() + j);
}

const double& MatrixD::at(const size_t i, const size_t j) const {
    if (j >= cols()) {
        ElemUtils::Formatter formatter;
        formatter
                << "Matrix column index out of bounds! Must be strictly inferior to "
                << cols() << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }
    if (i >= rows()) {
        ElemUtils::Formatter formatter;
        formatter
                << "Matrix row index out of bounds! Must be strictly inferior to "
                << rows() << ".";
        throw ElemUtils::CustomException("MatrixD", __func__, formatter.str());
    }
    return m_matrix[i * cols() + j];
}

} /* namespace NumA */
