/*
 * MatrixComplex3D.cpp
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#include "../../../../include/NumA/linear_algebra/matrix_complex/MatrixComplex3D.h"

#include <stdexcept>

namespace NumA {

MatrixComplex3D::MatrixComplex3D() :
        MatrixComplexD(3, 3) {
}

MatrixComplex3D::MatrixComplex3D(const MatrixComplex3D& v) :
        MatrixComplexD(v) {
}

MatrixComplex3D::MatrixComplex3D(const MatrixComplexD& v) :
        MatrixComplexD(v) {

    if (v.GetNElements(X) != 3 && v.GetNElements(Y) != 3)
        throw std::runtime_error(
                "MatrixComplex3D::MatrixComplex3D(): Can not copy from matrix of dimension different then 3x3");
}

MatrixComplex3D::~MatrixComplex3D() {
}

void MatrixComplex3D::operator =(const MatrixComplex3D& rhs) {
    this->MatrixComplexD::operator =(rhs);
}

MatrixComplex3D MatrixComplex3D::operator +(const MatrixComplex3D& rhs) {
    return MatrixComplex3D(this->MatrixComplexD::operator +(rhs));
}

MatrixComplex3D MatrixComplex3D::operator -(const MatrixComplex3D& rhs) {
    return MatrixComplex3D(this->MatrixComplexD::operator -(rhs));
}

MatrixComplex3D MatrixComplex3D::operator *(const double& rhs) {
    return MatrixComplex3D(this->MatrixComplexD::operator *(rhs));
}

MatrixComplex3D operator *(const double& lhs, MatrixComplex3D& rhs) {
    return rhs * lhs;
}

MatrixComplex3D MatrixComplex3D::operator *(const std::complex<double>& rhs) {
    return MatrixComplex3D(this->MatrixComplexD::operator *(rhs));
}

MatrixComplex3D operator *(const std::complex<double>& lhs,
        MatrixComplex3D& rhs) {
    return rhs * lhs;
}

} /* namespace NumA */
