/*
 * MatrixComplex4D.cpp
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#include "../../../../include/NumA/linear_algebra/matrix_complex/MatrixComplex4D.h"

#include <stdexcept>

namespace NumA {

MatrixComplex4D::MatrixComplex4D() :
        MatrixComplexD(4, 4) {
}

MatrixComplex4D::MatrixComplex4D(const MatrixComplex4D& v) :
        MatrixComplexD(v) {
}

MatrixComplex4D::MatrixComplex4D(const MatrixComplexD& v) :
        MatrixComplexD(v) {

    if (v.GetNElements(X) != 4 && v.GetNElements(Y) != 4)
        throw std::runtime_error(
                "MatrixComplex4D::MatrixComplex4D(): Can not copy from matrix of dimension different then 4x4");
}

MatrixComplex4D::~MatrixComplex4D() {
}

void MatrixComplex4D::operator =(const MatrixComplex4D& rhs) {
    this->MatrixComplexD::operator =(rhs);
}

MatrixComplex4D MatrixComplex4D::operator +(const MatrixComplex4D& rhs) {
    return MatrixComplex4D(this->MatrixComplexD::operator +(rhs));
}

MatrixComplex4D MatrixComplex4D::operator -(const MatrixComplex4D& rhs) {
    return MatrixComplex4D(this->MatrixComplexD::operator -(rhs));
}

MatrixComplex4D MatrixComplex4D::operator *(const double& rhs) {
    return MatrixComplex4D(this->MatrixComplexD::operator *(rhs));
}

MatrixComplex4D operator *(const double& lhs, MatrixComplex4D& rhs) {
    return rhs * lhs;
}

MatrixComplex4D MatrixComplex4D::operator *(const std::complex<double>& rhs) {
    return MatrixComplex4D(this->MatrixComplexD::operator *(rhs));
}

MatrixComplex4D operator *(const std::complex<double>& lhs,
        MatrixComplex4D& rhs) {
    return rhs * lhs;
}

} /* namespace NumA */
