/*
 * VectorComplexD.cpp
 *
 *  Created on: Jan 16, 2016
 *      Author: Pawel Sznajder (IPNO)
 */

#include "../../../../include/NumA/linear_algebra/vector_complex/VectorComplexD.h"

#include <cmath>
#include <stdexcept>

namespace NumA {

VectorComplexD::VectorComplexD() {

    throw std::runtime_error(
            "VectorComplexD::VectorComplexD(): Constructor not supported");

    m_n = 0;
    m_elements.clear();
}

VectorComplexD::VectorComplexD(const unsigned int n) {

    if (n == 0)
        throw std::runtime_error(
                "VectorComplexD::VectorComplexD(): Can not set vector of dimension equals zero");

    this->m_n = n;
    this->m_elements = std::vector<std::complex<double> >(n, std::complex<double>(0., 0.));
}

VectorComplexD::VectorComplexD(const VectorComplexD& v) {

    this->m_n = v.m_n;
    this->m_elements = v.m_elements;
}

VectorComplexD::~VectorComplexD() {
}

double VectorComplexD::Mag() const{
    return sqrt(std::real(this->Dot(this->Conjunct())));
}

VectorComplexD VectorComplexD::Conjunct() const{

    VectorComplexD r(this->m_n);

    for (unsigned int i = 0; i < this->m_n; i++)
        r.SetElement(i, std::conj(this->m_elements.at(i)));

    return r;
}

std::complex<double> VectorComplexD::Dot(const VectorComplexD& v) const{

    this->CompareNElements(v);

    std::complex<double> r(0., 0.);

    for (unsigned int i = 0; i < this->m_n; i++)
        r += this->m_elements.at(i) * v.m_elements.at(i);

    return r;
}

void VectorComplexD::Clear() {

    for (unsigned int i = 0; i < this->m_n; i++)
        this->SetElement(i, std::complex<double>(0., 0.));
}

void VectorComplexD::operator =(const VectorComplexD& rhs) {

    if (this == &rhs)
        return;

    this->CompareNElements(rhs);

    for (unsigned int i = 0; i < this->m_n; i++)
        this->SetElement(i, rhs.m_elements.at(i));
}

VectorComplexD VectorComplexD::operator +(const VectorComplexD& rhs) {

    this->CompareNElements(rhs);

    VectorComplexD r(this->m_n);

    for (unsigned int i = 0; i < this->m_n; i++)
        r.SetElement(i, this->m_elements.at(i) + rhs.m_elements.at(i));

    return r;
}

VectorComplexD VectorComplexD::operator -(const VectorComplexD& rhs) {

    this->CompareNElements(rhs);

    VectorComplexD r(this->m_n);

    for (unsigned int i = 0; i < this->m_n; i++)
        r.SetElement(i, this->m_elements.at(i) - rhs.m_elements.at(i));

    return r;
}

VectorComplexD VectorComplexD::operator *(const double& rhs) {

    VectorComplexD r(this->m_n);

    for (unsigned int i = 0; i < this->m_n; i++)
        r.SetElement(i, rhs * (this->m_elements.at(i)));

    return r;
}

VectorComplexD operator *(const double& lhs, VectorComplexD& rhs) {
    return rhs * lhs;
}

VectorComplexD VectorComplexD::operator *(const std::complex<double>& rhs) {

    VectorComplexD r(this->m_n);

    for (unsigned int i = 0; i < this->m_n; i++)
        r.SetElement(i, rhs * (this->m_elements.at(i)));

    return r;
}

VectorComplexD operator *(const std::complex<double>& lhs,
        VectorComplexD& rhs) {
    return rhs * lhs;
}

unsigned int VectorComplexD::GetNElements() const {
    return this->m_n;
}

void VectorComplexD::SetElement(const unsigned int i, const std::complex<double>& v) {

    if (i >= this->m_n)
        throw std::runtime_error("VectorComplexD::SetElement(): Illegal index");

    this->m_elements.at(i) = v;
}

std::complex<double> VectorComplexD::GetElement(const unsigned int i) const {

    if (i >= this->m_n)
        throw std::runtime_error("VectorComplexD::GetElement(): Illegal index");

    return this->m_elements.at(i);
}

void VectorComplexD::CompareNElements(const VectorComplexD& v) const{

    if (this->m_n != v.GetNElements())
        throw std::runtime_error(
                "VectorComplexD::CompareDimension(): Two complex vectors have different dimensions");
}

} /* namespace NumA */

