#include "../../../include/NumA/root_finding/NewtonMD.h"

#include <sstream>
#include <stdexcept>

namespace NumA {

NewtonMD::NewtonMD(int N, LinAlgUtils::Method linMethod) :
        m_N(N), m_linMethod(linMethod) {
}

NewtonMD::~NewtonMD() {
    // TODO Auto-generated destructor stub
}

int NewtonMD::getN() const {
    return m_N;
}

void NewtonMD::setN(int n) {
    m_N = n;
}

LinAlgUtils::Method NewtonMD::getLinMethod() const {
    return m_linMethod;
}

void NewtonMD::setLinMethod(LinAlgUtils::Method linMethod) {
    m_linMethod = linMethod;
}

VectorD NewtonMD::iterate(const VectorD & X0, const VectorD & G_X0,
        const MatrixD & J_G_X0) {
    if (X0.size() != getN() || G_X0.size() != getN() || J_G_X0.cols() != getN()
            || J_G_X0.rows() != getN()) {
        std::stringstream formatter;
        formatter
                << "[NewtonMD::iterate] Vectors or matrix don't have the right size: "
                << getN();
        throw std::runtime_error(formatter.str());
    }

    VectorD X = LinAlgUtils::solve(J_G_X0, G_X0, getLinMethod());
    VectorD X1 = X0 - X;
    return X1;
}

} /* namespace NumA */
