#include "../../../include/NumA/utils/Tolerances.h"

#include <sstream>
#include <stdexcept>

namespace NumA {

Tolerances::Tolerances() :
        m_absoluteTolerance(0.), m_relativeTolerance(0.) {
}

Tolerances::Tolerances(double absoluteTolerance, double relativeTolerance) :
        m_absoluteTolerance(0.), m_relativeTolerance(0.) {
    setAbsoluteTolerance(absoluteTolerance);
    setRelativeTolerance(relativeTolerance);
}

Tolerances::~Tolerances() {
}

double Tolerances::getAbsoluteTolerance() const {
    return m_absoluteTolerance;
}

void Tolerances::setAbsoluteTolerance(double absoluteTolerance) {
    testPositivity(absoluteTolerance);
    m_absoluteTolerance = absoluteTolerance;
}

double Tolerances::getRelativeTolerance() const {
    return m_relativeTolerance;
}

void Tolerances::setRelativeTolerance(double relativeTolerance) {
    testPositivity(relativeTolerance);
    m_relativeTolerance = relativeTolerance;
}

std::string Tolerances::toString() const {
    std::stringstream sstring;
    sstring << "Absolute tolerance = " << m_absoluteTolerance << '\n' << "Relative tolerance = "
            << m_relativeTolerance;
    return sstring.str();
}

void Tolerances::testPositivity(const double& tolerance) const {
    if (tolerance < 0.) {
        std::stringstream errorMessage;
        errorMessage
                << "(Tolerances::testPositivity) Tolerance should be >0. Here tolerance = "
                << tolerance;
        throw std::range_error(errorMessage.str());
    }
}

} // namespace NumA
